#!/usr/local/bin/perl -w

@sets = ("crisGet", "crisPut", "sisGet", "sisPut", "utilities");
@languages = ("c", "idl");

foreach $set (@sets) {
    undef(%val);

    $orderFile = "routineDBs/$set/order.txt";
    if (-e $orderFile) { open(ORDER, $orderFile) || die $!; }
    else { next; }

    $/ = "\n\n";	# read paragraph mode

    @sections = ();
    @routineIDs = ();
    while (<ORDER>) {
	@lines = split(/\n/);
	$section = $lines[0];
	push(@sections, $section);
	$i = 1;
	$description = "";
	while ($lines[$i] =~ /^\t/) {
	    ($trimmed) = ($lines[$i] =~ /^\t(.*)/);
	    $description .= $trimmed . " ";
	    $i++;
	}
	chop($description);
	$descriptions{$section} = $description;

	$routineIDs{$section} = "";
	for (; $i < @lines; $i++) {
	    $routineID = $lines[$i];
	    $routineIDs{$section} .= $routineID . "|";
	    foreach $lang (@languages) {
		$file = "routineDBs/$set/$routineID-$lang.db";
		if (-e $file) { open(DB, $file) || die $!; }
		else { next; }

		while (<DB>) {
		    @dbLines = split(/\n/, $_);
		    $key = shift(@dbLines);
		    $val{"$routineID-$lang-$key"} = join("\n", @dbLines);
		}
		@stats = stat(DB);
		close(DB);

		@dateFields = localtime($stats[9]);
		$mday = $dateFields[3];
		$month = $dateFields[4] + 1;
		$year = $dateFields[5] + 1900;
		$lastModifiedDate{"$routineID-$lang"} = "$month/$mday/$year";
	    }
	}
	chop($routineIDs{$section});
    }
    close(ORDER);

    foreach $lang (@languages) {
	open(HTML, ">$set-$lang.html") || die $!;
	open(HTINC, ">$set-$lang-contents.htinc") || die $!;

	print HTML <<"STOP_PRINT";

<HTML>
<HEAD>
<TITLE>ACEware Level1 Library: $lang $set</TITLE>
</HEAD>
<BODY BGCOLOR="#ffffff" TEXT="#000000">
<H1 ALIGN=CENTER>ACEware Level1 Library: $lang $set</H1>
<HR SIZE=5 NOSHADE>
<H3 ALIGN=CENTER><A NAME="contents">Contents</A></H3>
<CENTER>
<TABLE BORDER=0>

STOP_PRINT

	print HTINC <<"STOP_PRINT";

<H2 ALIGN=CENTER>
<A HREF="$set-$lang.html">ACEware Level1 Library: $lang $set</A>
</H2>
<H3 ALIGN=CENTER>Contents</H3>
<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0 WIDTH="100%">
STOP_PRINT

	for ($i = 0; $i < @sections; $i++) {
	    $section = $sections[$i];
	    print HTML <<"STOP_PRINT";

  <TR><TD ALIGN=CENTER COLSPAN=2><B>$section</B></TD></TR>
STOP_PRINT

	    if (!($i % 2)) { print HTINC "<TR>\n"; }
	    print HTINC <<"STOP_PRINT";
<TD ALIGN=CENTER>
<FONT SIZE="-1">
<FORM METHOD=POST ACTION="dummy">
<SELECT NAME="dummy" SIZE=1>
<OPTION> $section
<OPTION> --------------------
STOP_PRINT

	    @routineIDs = split(/\|/, $routineIDs{$section});
	    while ($routineID = shift(@routineIDs)) {
		$name = $val{"$routineID-$lang-_NAME_"};
		unless ($name) { next; }
		$date = $lastModifiedDate{"$routineID-$lang"};
		print HTML <<"STOP_PRINT";

  <TR><TD><A HREF="#$name">$name</A></TD><TD>$date</TD></TR>
STOP_PRINT

		print HTINC "<OPTION> $name\n";
	    }
	    print HTML <<"STOP_PRINT";

  <TR><TD COLSPAN=2></TD></TR>
STOP_PRINT

	    print HTINC "</SELECT></FORM></FONT></TD>\n";
	    if ($i % 2) { print HTINC "</TR>\n"; }
	}
	print HTINC "</TABLE>\n";
	close(HTINC);
	print HTML <<"STOP_PRINT";

</TABLE>
</CENTER>
<HR SIZE=5 NOSHADE>
STOP_PRINT

	for ($i = 0; $i < @sections; $i++) {
	    $section = $sections[$i];
	    print HTML <<"STOP_PRINT";

<H3 ALIGN=CENTER>$section</H3>
<BLOCKQUOTE>
$descriptions{$section}
</BLOCKQUOTE>
<HR WIDTH="80%" NOSHADE>
STOP_PRINT

	    @routineIDs = split(/\|/, $routineIDs{$section});
	    while ($routineID = shift(@routineIDs)) {
		unless ($val{"$routineID-$lang-_NAME_"}) { next; }
		$prototype = $val{"$routineID-$lang-_PROTOTYPE_"};
		$name = $val{"$routineID-$lang-_NAME_"};
		$prototype =~ s/$name/<FONT COLOR=\"#ff0000\">$name<\/FONT>/;

		$description = $val{"$routineID-$lang-_DESCRIPTION_"};
		$description =~ s/^_$/<P>/mg;

		$input = $val{"$routineID-$lang-_INPUT_"};
		$input =~ s/^\* /\n  <LI> /mg;
		$output = $val{"$routineID-$lang-_OUTPUT_"};
		$output =~ s/^\* /\n  <LI> /mg;

		print HTML <<"STOP_PRINT";

<A NAME="$name"><B>$val{"$routineID-$lang-_SHORT-DESCRIPTION_"}</B></A>
<PRE>
$prototype
</PRE>
<P>
$description
<P>
<B>Input:</B>
<UL>
$input
</UL>
<B>Output:</B>
<UL>
$output
</UL>

<CENTER>
<TABLE WIDTH="80%" BORDER=0>
  <TR><TD ALIGN="LEFT">
<FONT COLOR="#ff0000" SIZE="-1">
Last Modified: $lastModifiedDate{"$routineID-$lang"}
</FONT>
  </TD>
  <TD ALIGN="RIGHT">
<FONT SIZE="-1"><A HREF="#contents">Return to Contents</A></FONT>
  </TD></TR>
</TABLE>
</CENTER>
STOP_PRINT

		if (@routineIDs > 0) { print HTML "<HR WIDTH=\"80%\">\n"; }
	    }

	    if (@sections > 0) { print HTML "<HR NOSHADE>\n"; }
	}

	print HTML <<"STOP_PRINT";

</BODY>
</HTML>
STOP_PRINT

	close(HTML);
    }
}
