#include <stdio.h>
#include <string.h>
#include <time.h>
#include "hdfi.h"
#include "sis.h"
#include "cris.h"
#include "sisGetData.h"
#include "crisGetData.h"
#include "crisSisGetUtilities.h"
#include "utilities.h"
 
    /* local helper routines */
void changeToCStyleName(char *structName);

int main(int argc, char *argv[]) {
    uint32 secondsSince1996Start, secondsSince1996Stop, bytesToWrite;
    char *structName;
    void *pStructs, *pStructsStart;
    int count, structSize, id, fd;
    char tmpFileName[256];

    if (argc != 4) {
	fprintf(stderr, "getStructsStandAlone: incorrect number of");
	fprintf(stderr, " arguments.  Expected 3, got %d\n\n.", argc - 1);
	fprintf(stderr, "Exiting.\n\n");
	exit(1);
    }

    secondsSince1996Start = (uint32) atol(argv[1]);
    secondsSince1996Stop = (uint32) atol(argv[2]);
    structName = (char *) argv[3];
    changeToCStyleName(structName);

    structSize = getStructSizeFromName(structName);

    pStructs =
	getStructs(secondsSince1996Start, secondsSince1996Stop, structName);
    pStructsStart = pStructs;

    count = 0;
    while (getTimeStamp(pStructs, structName)) {
	count++;
	pStructs += structSize;
    }
    count++;				/* include zeroed struct */
    /* printf("count = %d\n", count); */

    bytesToWrite = ((uint32) count) * structSize;
    /* printf("bytesToWrite = %d\n", bytesToWrite); */

    id = 0;
    do {
	sprintf(tmpFileName, "get_structs_tmp.%d", id);
	id++;
    } while (fileExists(tmpFileName));

    fd = openOrCreateFileToReadWrite(tmpFileName);
    writeToFile(fd, (char *) pStructsStart, bytesToWrite);
    close(fd);

    printf("%s\n", tmpFileName);

    return(0);
}

    /* helper routines */

void changeToCStyleName(char *structName) {
    if (strstr(structName, "cris")) {
	if (!strcmp(structName, "browsecris"))
				strcpy(structName, "BrowseCris");
	else if (!strcmp(structName, "l1criscommandecho"))
				strcpy(structName, "L1CrisCommandEcho");
	else if (!strcmp(structName, "l1criscommandtable"))
				strcpy(structName, "L1CrisCommandTable");
	else if (!strcmp(structName, "l1crisdiagnosticevent"))
				strcpy(structName, "L1CrisDiagnosticEvent");
	else if (!strcmp(structName, "l1criseventcycle"))
				strcpy(structName, "L1CrisEventCycle");
	else if (!strcmp(structName, "l1crishighpriorityrate"))
				strcpy(structName, "L1CrisHighPriorityRate");
	else if (!strcmp(structName, "l1crishskp"))
				strcpy(structName, "L1CrisHskp");
	else if (!strcmp(structName, "l1crislowpriorityrate"))
				strcpy(structName, "L1CrisLowPriorityRate");
	else if (!strcmp(structName, "l1crissubset"))
				strcpy(structName, "L1CrisSubset");
	else if (!strcmp(structName, "l1crissummary"))
				strcpy(structName, "L1CrisSummary");
    }
    else if (strstr(structName, "sis")) {
	if (!strcmp(structName, "browsesis"))
				strcpy(structName, "BrowseSis");
	else if (!strcmp(structName, "l1siscommandecho"))
				strcpy(structName, "L1SisCommandEcho");
	else if (!strcmp(structName, "l1siscommandtable"))
				strcpy(structName, "L1SisCommandTable");
	else if (!strcmp(structName, "l1sisdacoffset0"))
				strcpy(structName, "L1SisDacOffset0");
	else if (!strcmp(structName, "l1sisdacoffset1"))
				strcpy(structName, "L1SisDacOffset1");
	else if (!strcmp(structName, "l1sisdiagnosticevent"))
				strcpy(structName, "L1SisDiagnosticEvent");
	else if (!strcmp(structName, "l1siseventcycle"))
				strcpy(structName, "L1SisEventCycle");
	else if (!strcmp(structName, "l1sishighpriorityrate"))
				strcpy(structName, "L1SisHighPriorityRate");
	else if (!strcmp(structName, "l1sishskp"))
				strcpy(structName, "L1SisHskp");
	else if (!strcmp(structName, "l1sislowpriorityrate"))
				strcpy(structName, "L1SisLowPriorityRate");
	else if (!strcmp(structName, "l1sissubset"))
				strcpy(structName, "L1SisSubset");
	else if (!strcmp(structName, "l1sissummary"))
				strcpy(structName, "L1SisSummary");

    }
    else {
	    /* if we got here, did not recognize the structure name */
	fprintf(stderr, "getStructsStandAlone: ERROR\n");
	fprintf(stderr, "  Struct name ->%s<- not recognized.  Exiting.\n\n",
								structName);
	exit(0);
    }
}
