#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "hdfi.h"
#include "cris.h"
#include "FileTypes.h"
#include "sortTracker.h"
#include "crisPutData.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*

*/

    /** struct type dependent code **/
    /* static used here to make these variables internal to this file */
static unsigned int DATA_TYPE_ID = FILE_CRIS_DIAGNOSTIC_EVENT;
static char dataBuffer[sizeof(struct L1CrisDiagnosticEvent)];
static char FILE_TYPE_SUFFIX[] = "de";

static void reorderFile(int fd, char *path);
int dataBlocksAreEqual(int fd, long structLocationInFile,
						long struct2LocationInFile);

void putCrisDiagnosticEventGroup(struct L1CrisDiagnosticEvent *pDataStruct) {
    int fd = -1, intervalNumber;
    int structsAreOutOfOrderInFile = 0;
    char path[256];
    uint32 dayBeginSecond1996 = 0, dayEndSecond1996 = 0, sec1996;

    while (pDataStruct->Second1996MinorFrame0 > 0) {
	sec1996 = pDataStruct->Second1996MinorFrame0;

	    /* is next Second1996MinorFrame0 outside the current day? */
	if ((sec1996 < dayBeginSecond1996) || (sec1996 > dayEndSecond1996)) {
	    unsigned int day1996;
	    struct tm *pDate;

		/* if this is not the initial file opening, we must first */
		/* deal with closing up and reordering, if need be, the */
		/* current file. */
	    if (dayBeginSecond1996 != 0) {
		/*if (checkIfStructsAreOutOfOrderInTheFile(fd))*/
		if (structsAreOutOfOrderInFile) reorderFile(fd, path);
		close(fd);
	    }
	    structsAreOutOfOrderInFile = 0;

	    day1996 = getDay1996FromSecondsSince1996(sec1996);
	    pDate = getDateFromDay1996(day1996);
	    dayBeginSecond1996 = getSecondsSince1996FromDate(pDate);
	    dayEndSecond1996 = dayBeginSecond1996 + (24 * 3600) - 1;

	    determineDataFilePathAndCreateDirsAsNeeded(sec1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'c');
	    if ((fd = open(path, O_RDWR)) == -1) { /* file doesn't exist yet? */
		fd = openOrCreateFileToReadWrite(path);
		writeHeader(fd, DATA_TYPE_ID);
		    /* since this is a new file, write struct and continue */
		writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));

		    /* is there a continuation of this data block? */
		while (pDataStruct->ContinuationFlag) {
		    pDataStruct++;
			/* check that time tags are identical.  if not, exit. */
		    if (pDataStruct->Second1996MinorFrame0 != sec1996) {
			fprintf(stderr, "ContinuationFlag set on previous ");
			fprintf(stderr,"struct (time tagged as %ld)\n",sec1996);
			fprintf(stderr, "but this struct has a time tag ");
			fprintf(stderr, "of %ld\n\nExiting.\n\n",
					pDataStruct->Second1996MinorFrame0);
			exit(1);
		    }
			/* else write the continuation to file */
		    writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
		}

		pDataStruct++;
		continue;	/* next struct please... */
	    }
	}

	    /* file already exists.  first read the last struct in the file */
	    /* and see if we just need to append passed struct to file's end */

	    /* go to end of file, and back up one struct size */
	lseek(fd, -1 * sizeof(dataBuffer), 2);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	    /* is new struct's time stamp before last struct in file's */
	    /*     time stamp?  if so, flag it for work later. */
	    /** struct type dependent code **/
	if (sec1996 <= ((struct L1CrisDiagnosticEvent *)
				dataBuffer)->Second1996MinorFrame0)
						structsAreOutOfOrderInFile = 1;

	    /* append passed struct whether in order or not */
	lseek(fd, 0L, 2);   /* go to end of file */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));

	    /* is there a continuation of this data block? */
	while (pDataStruct->ContinuationFlag) {
	    pDataStruct++;
		/* check that time tags are identical.  if not, exit. */
	    if (pDataStruct->Second1996MinorFrame0 != sec1996) {
		fprintf(stderr, "ContinuationFlag set on previous ");
		fprintf(stderr, "struct (time tagged as %ld)\n",sec1996);
		fprintf(stderr, "but this struct has a time tag ");
		fprintf(stderr, "of %ld\n\nExiting.\n\n", 
                                        pDataStruct->Second1996MinorFrame0);
		exit(1);
	    }
		/* else write the continuation to file */
	    writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
	}

	pDataStruct++;
    }
	/* note: reorderFile closes fd */
    if (structsAreOutOfOrderInFile) reorderFile(fd, path);
    else close(fd);
}

    /* this is used to reorder the data file by time stamp */
    /* it is also called by itself (i.e., recursively) to reorder */
    /* any overlap file which is created/written to */
    /* static is used to hide this function outside this file */

static void reorderFile(int fd, char *path) {
	/** struct type dependent code **/
    struct L1CrisDiagnosticEvent *pStruct;

    int TRUE = 1, FALSE = 0;
    int tempFd, i, reorderingOverlapFile;
    int overlapFileNeedsReordering = FALSE;
    unsigned int structCount;
    char tempPath[256], overlapFilePath[256];
    struct stat fileStats;
	/* static so we know what to free on next call */
    static struct sortTracker *pSortTracker;
    struct sortTracker *pSTHolder;
    long structLocationInFile, struct2LocationInFile;
    uint32 previousTimeStamp;

    free(pSortTracker);

	/* check to see if we are reordering the overlap file here */
	/* we want to make sure we don't start making .overlap.overlap files */
    if (strstr(path, ".overlap") == NULL) {
	    /* this means we will be appending any overlapping data */
	    /* to the overlap file */
	reorderingOverlapFile = FALSE;
	strcpy(overlapFilePath, path);              /* may or may not */
	strcat(overlapFilePath, ".overlap");        /*     be used... */
    }
	/* this means it is the overlap file itself we are reordering, so */
	/* we will just write all structs to the overlap file, regardless */
	/* of overlapping time stamps */
    else reorderingOverlapFile = TRUE;
    /* printf("reorderingOverlapFile = %d\n", reorderingOverlapFile); */

	/* determine correct ordering of structs in file */
    fstat(fd, &fileStats);
    if (reorderingOverlapFile)
			structCount = fileStats.st_size / sizeof(dataBuffer);
    else structCount = (fileStats.st_size - HEADER_BYTES) / sizeof(dataBuffer);
    /* printf("structCount = %d\n", structCount); */
    pSortTracker = (struct sortTracker *)
				calloc(structCount, sizeof(struct sortTracker));
    for (i = 0; i < structCount; i++) {
	long locationToSeek;
	if (reorderingOverlapFile) locationToSeek = (sizeof(dataBuffer) * i);
	else locationToSeek = HEADER_BYTES + (sizeof(dataBuffer) * i);
	lseek(fd, locationToSeek, 0);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	    /** struct type dependent code **/
	pStruct = (struct L1CrisDiagnosticEvent *) dataBuffer;

	(pSortTracker + i)->Second1996 = pStruct->Second1996MinorFrame0;
	(pSortTracker + i)->structID = i;

	    /* zero time stamps from continuation structs to mark them */
	    /* as meaningless in the sort to come */
	while (pStruct->ContinuationFlag) {
	    i++;
	    /* printf("zeroing (pSortTracker + %d)->Second1996:continuation\n",
									i); */
	    (pSortTracker + i)->Second1996 = 0;
	    (pSortTracker + i)->structID = i;
	    if (reorderingOverlapFile) locationToSeek = (sizeof(dataBuffer)*i);
	    else locationToSeek = HEADER_BYTES + (sizeof(dataBuffer) * i);
	    lseek(fd, locationToSeek, 0);
	    readFromFile(fd, dataBuffer, sizeof(dataBuffer));
	    pStruct = (struct L1CrisDiagnosticEvent *) dataBuffer;
	}
    }

    pSTHolder = pSortTracker;
    qsort((void *) pSortTracker, structCount, sizeof(struct sortTracker),
								sortTrackers);
    pSortTracker = pSTHolder;

	/* check for identical time stamps on the starting struct of any */
	/* data blocks. */
    for (i = 0; i < structCount - 1; i++) {
	/* printf("after sort: (pSortTracker + %d)->Second1996 = %d\n", i,
					(pSortTracker + i)->Second1996); */
	if ((pSortTracker + i)->Second1996 == 0) continue;
	if ((pSortTracker + i)->Second1996 == (pSortTracker + i+1)->Second1996){
	    structLocationInFile = HEADER_BYTES +
			(pSortTracker + i)->structID * sizeof(dataBuffer);
	    struct2LocationInFile = HEADER_BYTES +
			(pSortTracker + i + 1)->structID * sizeof(dataBuffer);
	    if (reorderingOverlapFile) {
		structLocationInFile -= HEADER_BYTES;
		struct2LocationInFile -= HEADER_BYTES;
	    }
	    if (dataBlocksAreEqual(fd, structLocationInFile,
						struct2LocationInFile)) {
	        /* printf("zeroing (pSortTracker + %d)->Second1996:identical\n",
									i); */
		(pSortTracker + i)->Second1996 = 0;
	    }
	}
    }
    /* printf("after sort: (pSortTracker + %d)->Second1996 = %d\n", i,
					(pSortTracker + i)->Second1996); */

	/* now write structs into a tmp file */
    strcpy(tempPath, path);
    strcat(tempPath, "-tmp");
    tempFd = openOrCreateFileToReadWrite(tempPath);
    if (!reorderingOverlapFile) writeHeader(tempFd, DATA_TYPE_ID);
    previousTimeStamp = 0;
    for (i = 0; i < structCount; i++) {
	/* printf("main:(pSortTracker + %d)->Second1996 = %d\n",
					i, (pSortTracker + i)->Second1996); */
	/* printf("pStruct->ContinuationFlag = %d\n",
						pStruct->ContinuationFlag); */
	if ((pSortTracker + i)->Second1996 == 0) continue;

	structLocationInFile = HEADER_BYTES +
			(pSortTracker + i)->structID * sizeof(dataBuffer);
	if (reorderingOverlapFile) structLocationInFile -= HEADER_BYTES;
	lseek(fd, structLocationInFile, 0);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	    /** struct type dependent code **/
	pStruct = (struct L1CrisDiagnosticEvent *) dataBuffer;

	    /* check for structs having the same time tags */
	if ((pStruct->Second1996MinorFrame0 == previousTimeStamp) &&
						!reorderingOverlapFile) {
		/* write the struct to an overlap file */
	    int overlapFd;

	    /* printf("\noverlap: writing pStruct->Second1996MinorFrame0"); */
	    /* printf(" = %d\n\n", pStruct->Second1996MinorFrame0); */
	    overlapFd = openOrCreateFileToReadWrite(overlapFilePath);
	    lseek(overlapFd, 0L, 2);   /* go to end of file */
	    writeToFile(overlapFd, dataBuffer, sizeof(dataBuffer));
	    previousTimeStamp = pStruct->Second1996MinorFrame0;
		/* get continuation structs as well */
	    while (pStruct->ContinuationFlag) {
		readFromFile(fd, dataBuffer, sizeof(dataBuffer));
		writeToFile(overlapFd, dataBuffer, sizeof(dataBuffer));
		pStruct = (struct L1CrisDiagnosticEvent *) dataBuffer;
	    }
	    close(overlapFd);
	    overlapFileNeedsReordering = TRUE;
	    continue;
	}
	else {
	    /* printf("\ntmp: writing pStruct->Second1996MinorFrame0"); */
	    /* printf(" = %d\n\n", pStruct->Second1996MinorFrame0); */
	    writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
	    previousTimeStamp = pStruct->Second1996MinorFrame0;
		/* get continuation structs as well */
	    while (pStruct->ContinuationFlag) {
		readFromFile(fd, dataBuffer, sizeof(dataBuffer));
		writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
		pStruct = (struct L1CrisDiagnosticEvent *) dataBuffer;
	    }
	}
    }
    close(tempFd);
    close(fd);

	/* finally, replace the old file with the new file */
    if (remove(path)) {
	fprintf(stderr, "Couldn't remove %s.\n\nExiting.\n\n", path);
	exit(0);
    }

    if (rename(tempPath, path)) {
	fprintf(stderr, "Couldn't rename %s to %s.\n\nExiting.\n\n",
								tempPath, path);
	exit(0);
    }

    if (overlapFileNeedsReordering) {
	int overlapFd;

	/* printf("Reordering overlap file now...\n"); */
	overlapFd = openOrCreateFileToReadWrite(overlapFilePath);
	reorderFile(overlapFd, overlapFilePath);
	    /* reorderFile closes overlapFd, no need to do it here */
    }
}

int dataBlocksAreEqual(int fd, long structLocationInFile,
						long struct2LocationInFile) {
         /** struct type dependent code **/
    struct L1CrisDiagnosticEvent struct1, struct2;

    int FALSE = 0, TRUE = 1;

    lseek(fd, structLocationInFile, 0);
    readFromFile(fd, (char *) &struct1, sizeof(struct1));
    lseek(fd, struct2LocationInFile, 0);
    readFromFile(fd, (char *) &struct2, sizeof(struct2));

	/* return false if these structs are not equal */
    if (memcmp(&struct1, &struct2, sizeof(struct1))) return FALSE;

	/* check any continuation structs */
   while (struct1.ContinuationFlag) {
	structLocationInFile += sizeof(struct1);
	struct2LocationInFile += sizeof(struct1);
	lseek(fd, structLocationInFile, 0);
	readFromFile(fd, (char *) &struct1, sizeof(struct1));
	lseek(fd, struct2LocationInFile, 0);
	readFromFile(fd, (char *) &struct2, sizeof(struct2));

	    /* return false if these structs are not equal */
	if (memcmp(&struct1, &struct2, sizeof(struct1))) return FALSE;
    }
	/* all checks have been made with no differences.  return true. */
    return TRUE;
}
