#include <stdio.h>
#include <time.h>
#include <string.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*

version 1.2:
    added passed parameter char instrumentID = 'c' or 's' for CRIS and
    SIS respectively, so same code can be used for both.

determineDataFilePathAndCreateDirsAsNeeded takes the number of seconds
since 00:00:00 1/1/96, a data type file suffix, and returns the path
for the data file (of the form .../s1996.01/s1996.01.02/s1996.01.02.evt)
and also the interval number that secondsSinceMidnight fits into in
the 256 second blocks that cover the day in question.

*/

void determineDataFilePathAndCreateDirsAsNeeded(uint32 secondsSince1996,
				const char *dataFileSuffix, char *path,
				int *pIntervalNumber, char instrumentID) {
    struct tm *pTime_str;
    int year, month, mday, secondsSinceMidnight;
    char yearMonthString[8], dateString[10];
    char *BASE_DIRECTORY, *pChar, *pToken;
    char baseDirShort[256], buffer[256];

    if ((instrumentID == 'C') || (instrumentID == 'c')) {	/* CRIS */
	instrumentID = 'c';
	BASE_DIRECTORY = getenv("L1_CRIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_CRIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n\nExiting.\n\n");
	    exit(1);
	}
    }
    else if ((instrumentID == 'S') || (instrumentID == 's')) {       /* SIS */
	instrumentID = 's';
	BASE_DIRECTORY = getenv("L1_SIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_SIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n\nExiting.\n\n");
	    exit(1);
	}
    }
    else {
	fprintf(stderr, "Invalid instrument ID: %c.\n\nExiting.\n\n",
								instrumentID);
	exit(1);
    }
    if (!directoryExists(BASE_DIRECTORY)) {
	fprintf(stderr, "The base directory %s does not exist.\n\nExiting.\n\n",
								BASE_DIRECTORY);
	exit(1);
    }

	/* use directories after the "data" directory in the environment */
	/* variable L1_[CRIS|SIS]_BASE_DIRECTORY as a file identifier to */
	/* be prepended to the data file name (baseDirShort refers to the */
	/* base directory, shortened by chopping of everything up to "data") */
    strcpy(buffer, BASE_DIRECTORY);
    pChar = buffer;
    strcpy(baseDirShort, "");
    while ((pToken = strtok(pChar, "/"))) {
	pChar = NULL;
	if (!strcmp(pToken, "data")) {	/* if (token == "data") */
	    while ((pToken = strtok(pChar, "/"))) {
		if (!strcmp(pToken, "data")) {
		    fprintf(stderr,
			"Directory \"data\" appears multiple times in base");
		    fprintf(stderr, "directory \"%s\"\n\nExiting.\n\n",
								BASE_DIRECTORY);
		    exit(1);
		}
		if (!strcmp(pToken, "")) break;		/* ignore nothing */
		if (*pToken == ' ') break;		/* ignore spaces */
		if (*pToken == '	') break;	/* ignore tabs */
		strcat(baseDirShort, pToken);
		strcat(baseDirShort, ".");
	    }
	    break;
	}
    }
    if (!strcmp(baseDirShort, "")) {
	fprintf(stderr, "\"data\" directory and/or its subdirectories not");
	fprintf(stderr, " found in base directory \"%s\"\n\nExiting.\n\n",
								BASE_DIRECTORY);
	exit(1);
    }
    /* fprintf(stderr, "baseDirShort = %s\n", baseDirShort); */

    pTime_str = getDateFromSecondsSince1996(secondsSince1996);
    year = pTime_str->tm_year + 1900;
    month = pTime_str->tm_mon + 1;
    mday = pTime_str->tm_mday;
    secondsSinceMidnight = pTime_str->tm_hour * 3600 +
				pTime_str->tm_min * 60 + pTime_str->tm_sec;
	/* secondsSinceMidnight = 0-255 -> block 0, 256-511 -> block 1, etc. */
	/* this is a return value for later use */
    *pIntervalNumber = secondsSinceMidnight / DATA_BLOCK_INTERVAL;
 
    sprintf(yearMonthString, "%4.4u.%2.2u", year, month);
    sprintf(path, "%s/%s", BASE_DIRECTORY, yearMonthString);
    if (!directoryExists(path)) {
	/* fprintf(stderr, "directory %s does not exist\n", path); */
	createDirectory(path);
    }
 
    sprintf(dateString, "%4.4u.%2.2u.%2.2u", year, month, mday);
    sprintf(path, "%s/%s/%s", BASE_DIRECTORY, yearMonthString, dateString);
    if (!directoryExists(path)) {
	/* fprintf(stderr, "directory %s does not exist\n", path); */
	createDirectory(path);
    }
 
    sprintf(path, "%s/%s/%s/%s%s.%s", BASE_DIRECTORY,
	yearMonthString, dateString, baseDirShort, dateString, dataFileSuffix);
    /* printf("path = %s\n", path); */
}
