#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "hdfi.h"
#include "sis.h"
#include "sortTracker.h"
#include "FileTypes.h"
#include "sisPutData.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*

*/

    /** struct type dependent code **/
    /* static used here to make these variables internal to this file */
static unsigned int DATA_TYPE_ID = FILE_SIS_DIAGNOSTIC_EVENT;
static char FILE_TYPE_SUFFIX[] = "de";
static char dataBuffer[sizeof(struct L1SisDiagnosticEvent)];

static void reorderFile(int fd, char *path);

void putSisDiagnosticEventGroup(struct L1SisDiagnosticEvent *pDataStruct) {
    int fd = -1, intervalNumber;
    int structsAreOutOfOrderInFile = 0;
    char path[256];
    uint32 dayBeginSecond1996 = 0, dayEndSecond1996 = 0, sec1996;

    while (pDataStruct->Second1996MinorFrame0 > 0) {
	sec1996 = pDataStruct->Second1996MinorFrame0;

	    /* is next Second1996MinorFrame0 outside the current day? */
	if ((sec1996 < dayBeginSecond1996) || (sec1996 > dayEndSecond1996)) {
	    unsigned int day1996;
	    struct tm *pDate;

		/* if this is not the initial file opening, we must first */
		/* deal with closing up and reordering, if need be, the */
		/* current file. */
	    if (dayBeginSecond1996 != 0) {
		if (structsAreOutOfOrderInFile) reorderFile(fd, path);
		close(fd);
	    }
	    structsAreOutOfOrderInFile = 0;

	    day1996 = getDay1996FromSecondsSince1996(sec1996);
	    pDate = getDateFromDay1996(day1996);
	    dayBeginSecond1996 = getSecondsSince1996FromDate(pDate);
	    dayEndSecond1996 = dayBeginSecond1996 + (24 * 3600) - 1;

	    determineDataFilePathAndCreateDirsAsNeeded(sec1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'S');
	    if ((fd = open(path, O_RDWR)) == -1) { /* file doesn't exist yet? */
		fd = openOrCreateFileToReadWrite(path);
		writeHeader(fd, DATA_TYPE_ID);
		    /* since this is a new file, write struct and continue */
		writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
		pDataStruct++;
		continue;	/* next struct please... */
	    }
	}

	    /* file already exists.  first read the last struct in the file */
	    /* and see if we just need to append passed struct to file's end */

	    /* go to end of file, and back up one struct size */
	lseek(fd, -1 * sizeof(dataBuffer), 2);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	    /* is new struct's time stamp before last struct in file's */
	    /*     time stamp?  if so, flag it for work later. */
	    /** struct type dependent code **/
	if (sec1996 <=
	    ((struct L1SisDiagnosticEvent *) dataBuffer)->Second1996MinorFrame0)
						structsAreOutOfOrderInFile = 1;

	    /* append passed struct whether in order or not */
	lseek(fd, 0L, 2);   /* go to end of file */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));

	pDataStruct++;
    }
    if (structsAreOutOfOrderInFile) reorderFile(fd, path);
    close(fd);
}

    /* this is used to reorder the data file by time stamp */
    /* it is also called by itself (i.e., recursively) to reorder */
    /* any overlap file which is created/written to */
    /* static is used to hide this function outside this file */

static void reorderFile(int fd, char *path) {
	/** struct type dependent code **/
    struct L1SisDiagnosticEvent *pStruct;
    struct L1SisDiagnosticEvent previousStruct;

    int tempFd, i;
    unsigned int structCount;
    char tempPath[256];
    struct stat fileStats;
	/* static to keep track of what to free */
    static struct sortTracker *pSortTracker;
    struct sortTracker *pSTHolder;
    long structLocationInFile;

    free(pSortTracker);

	/* determine correct ordering of structs in file */
    fstat(fd, &fileStats);
    structCount = (fileStats.st_size - HEADER_BYTES) / sizeof(dataBuffer);
    pSortTracker = (struct sortTracker *)
				calloc(structCount, sizeof(struct sortTracker));
    for (i = 0; i < structCount; i++) {
	lseek(fd, HEADER_BYTES + (sizeof(dataBuffer) * i), 0);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	    /** struct type dependent code **/
	pStruct = (struct L1SisDiagnosticEvent *) dataBuffer;

	(pSortTracker + i)->Second1996 = pStruct->Second1996MinorFrame0;
	(pSortTracker + i)->structID = i;
    }

    pSTHolder = pSortTracker;
    qsort((void *) pSortTracker, structCount, sizeof(struct sortTracker),
								sortTrackers);
    pSortTracker = pSTHolder;

	/* now write structs into a tmp file */
    strcpy(tempPath, path);
    strcat(tempPath, "-tmp");
    tempFd = openOrCreateFileToReadWrite(tempPath);
    writeHeader(tempFd, DATA_TYPE_ID);
    for (i = 0; i < structCount; i++) {
	    /** struct type dependent code **/
	structLocationInFile = HEADER_BYTES +
	    (pSortTracker + i)->structID * sizeof(struct L1SisDiagnosticEvent);

	lseek(fd, structLocationInFile, 0);
	readFromFile(fd, dataBuffer, sizeof(dataBuffer));

	if (i != 0) {
		/** struct type dependent code **/
	    pStruct = (struct L1SisDiagnosticEvent *) dataBuffer;

		/* check for structs having the same time tags */
	    if (previousStruct.Second1996MinorFrame0 ==
					pStruct->Second1996MinorFrame0) {
		    /* found two equal structs?  don't write it twice. */
		if (!memcmp(&previousStruct, dataBuffer, sizeof(dataBuffer)))
								continue;
		    /* else write the struct to an overlap file */
		else {
		    int overlapFd;

		    strcpy(tempPath, path);
		    strcat(tempPath, ".overlap");
		    overlapFd = openOrCreateFileToReadWrite(tempPath);
		    lseek(overlapFd, 0L, 2);   /* go to end of file */
		    writeToFile(overlapFd, dataBuffer, sizeof(dataBuffer));
		    close(overlapFd);
		    strcpy(tempPath, path);	/* reset path value */
		    strcat(tempPath, "-tmp");
		    continue;
		}
	    }
	}
	writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
	memcpy(&previousStruct, dataBuffer, sizeof(dataBuffer));
    }
    close(tempFd);
    close(fd);

	/* finally, replace the old file with the new file */
    if (remove(path)) {
	fprintf(stderr, "Couldn't remove %s.\n\nExiting.\n\n", path);
	exit(0);
    }

    if (rename(tempPath, path)) {
	fprintf(stderr, "Couldn't rename %s to %s.\n\nExiting.\n\n",
								tempPath, path);
	exit(0);
    }
}
