#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "hdfi.h"
#include "sis.h"
#include "FileTypes.h"
#include "sisPutData.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"
/*
version 1.3:
	change storage/initialization method to standard one for
	periodic data.

version 1.2:
	instead of merely appending new struct, create a new file
	placing the struct in the location which keeps the structs
	in order, time-wise.

version 1.1:
	change the storage of this data to be sequentially appended
	to file instead of placed in an interval.  this data is
	not coming in in a continuous stream as the others are.

version 1.0:
	initial version for putSisSubset.

putSisSubset is passed a pointer to a struct L1SisSubset
which it then attempts to archive inside a daily file in the sis data
archive directory tree in this manner:

a base data directory is gotten from the environment variable
L1_SIS_DATA_BASE_DIRECTORY.  if this environment variable has
not been defined, putSisSubset exits.  for following examples,
let's say this has been defined as /ace/sis/data.  if /ace/sis/data
does not exist, putSisSubset exits.

putSisSubset reads the space craft seconds since 1996 and determines
the year, month, and day.  for example's sake, say year = 1999,
month = 5 (1-12), and day = 23 (1-31).

putSisSubset then checks for the existence of a directory named
/ace/sis/data/S1999.05/, and if not finding it, creates it.  the
same for the next directory level, /ace/sis/data/S1999.05/S1999.05.23/

putSisSubset then checks for the existence of the file
/ace/sis/data/S1999.05/S1999.05.23/S1999.05.23.sub
if this file is not found, it is created and initialized with
an 8 word header. 

putSisSubset then reads what is already in this file.  if the
data struct is identical to another already in the file, nothing
is written to file, otherwise the data is inserted into the file
in time ordered sequence.  the return value in the case where an identical
data structure is found is 3.  otherwise putSisSubset
returns a 0, signalling the struct has been appended to the file
normally.

a return value of 1 was added in version 1.2 to signal that a
non-identical struct having the same time stamp as the passed
struct was found in the file.  in this case, the original file
is left unchanged, and the passed struct is written to an
overlap file (signified as same file name, appended by ".overlap").

*/

int putSisSubset(struct L1SisSubset *pDataStruct) {
	/** struct type dependent code **/
    unsigned int DATA_TYPE_ID = FILE_SIS_SUBSET;
    char FILE_TYPE_SUFFIX[] = "sub";
    unsigned int TIME_INTERVAL = 1;

	/* declared static so memory allocation occurs only once */
	/** struct type dependent code **/
    static char dataBuffer[sizeof(struct L1SisSubset)];

	/* generalized for all sis data structures beyond this point */
	/*     (assumes all such structs have an identical Second1996 field) */

    int fd, intervalNumber, fileInitialized, status;
    char path[256];
 
	/* this returns a value to intervalNumber as well as path */
    determineDataFilePathAndCreateDirsAsNeeded(pDataStruct->Second1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'S');

    if ((fd = open(path, O_RDWR)) == -1) {
	fileInitialized = 0;
	fd = openOrCreateFileToReadWrite(path);
    }
    else fileInitialized = 1;
 
    if (!fileInitialized) createInitializedFileWithIntervalN(fd,
		dataBuffer, sizeof(dataBuffer), DATA_TYPE_ID, TIME_INTERVAL);
 
    status = writeDataStructToFile(fd, intervalNumber, dataBuffer,
				sizeof(dataBuffer), (char *) pDataStruct, path);

    close(fd);
    return status;
}
